package eu.mhsl.minenet.minigames.handler.global;

import eu.mhsl.minenet.minigames.Main;
import eu.mhsl.minenet.minigames.api.QueuedPlayerRooms;
import eu.mhsl.minenet.minigames.instance.room.Room;
import eu.mhsl.minenet.minigames.instance.transfer.Transfer;
import eu.mhsl.minenet.minigames.message.Icon;
import eu.mhsl.minenet.minigames.message.type.ChatMessage;
import eu.mhsl.minenet.minigames.skin.SkinCache;
import eu.mhsl.minenet.minigames.util.MoveInstance;
import net.minestom.server.MinecraftServer;
import net.minestom.server.entity.Player;
import net.minestom.server.event.EventListener;
import net.minestom.server.event.player.PlayerLoginEvent;
import eu.mhsl.minenet.minigames.instance.hub.Hub;
import net.minestom.server.permission.Permission;
import net.minestom.server.timer.TaskSchedule;
import org.jetbrains.annotations.NotNull;
import org.spongepowered.configurate.serialize.SerializationException;

import java.util.Objects;
import java.util.UUID;
import java.util.logging.Logger;

public class PlayerLoginHandler implements EventListener<PlayerLoginEvent> {
    @Override
    public @NotNull Class<PlayerLoginEvent> eventType() {
        return PlayerLoginEvent.class;
    }

    @Override
    public @NotNull Result run(@NotNull PlayerLoginEvent event) {
        Player p = event.getPlayer();

        Transfer transferInstance = new Transfer();
        p.setRespawnPoint(transferInstance.getSpawn());
        event.setSpawningInstance(transferInstance);

        UUID pushQueue = QueuedPlayerRooms.pullQueue(event.getPlayer().getUuid());

        MinecraftServer.getSchedulerManager().scheduleTask(
                () -> {
                    if(pushQueue != null) {
                        Room.setRoom(p, Room.getRoom(pushQueue).orElseThrow());
                    } else {
                        MoveInstance.move(p, Hub.INSTANCE);
                    }
                },
                TaskSchedule.seconds(5),
                TaskSchedule.stop()
        );

        SkinCache.applySkin(p);

        try {
            if(Objects.requireNonNull(Main.globalConfig.node("admins").getList(String.class)).stream().anyMatch(s -> s.equalsIgnoreCase(p.getUsername()))) {
                p.addPermission(new Permission("admin"));
            }
        } catch (SerializationException | NullPointerException ignored) {}

        Logger.getLogger("user").info(p.getUsername() + " joined");

        new ChatMessage(Icon.SCIENCE).appendStatic("unübersetzter Jointext: ").appendStatic(p.getUsername()).send(MinecraftServer.getConnectionManager().getOnlinePlayers());

        return Result.SUCCESS;
    }
}
